/****************************************************************************/
/* FICHIER TRAJPOL.C : Procedures de calcul de trajectoires polynomiales    */
/****************************************************************************/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "trajpol.h"
/*--------------------------------------------------------------------------*/
/* Point d'abscisse parametrique u d'une courbe polynomiale                 */
/*--------------------------------------------------------------------------*/
void ptu_trajpol(double *pt, double u, double *coef, int dim, int degre)
/*----------------------------------------------------------------------------
    Entrees :
    coef    : Coefficients des polynomes presentes en vecteur :
            Coef[i*dim+k] est la composante k du coef de degre i
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome

    Sortie :
        pt      : Point calcule
*/
{
int k, id ;

    for (k = 0 , id = degre*dim ; k < dim ; k++)
    pt[k] = coef[id+k] ;

    while ((id -= dim) >= 0)
    {
    for (k = 0 ; k < dim ; k++)
        pt[k] = coef[id+k] + u * pt[k] ;
    }
} /**/
/*------------------------------------------------------------------------*/
void d1Mdu_trajpol (double *pt, double u, double *coef, int dim, int degre)
/*------------------------------------------------------------------------*/
/*
Derivee parametrique 1ere d'une courbe polynomiale
--------------------------------------------------
    Entree :
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome

    Sortie :
    pt      : derivee calculee
*/
{
int k, id ;
double a ;

    if (degre < 1)
    { puts ("Erreur : appel der1trajpol avec degre < 1") ; exit (1) ; }

    a = degre ;
    for (k = 0, id = degre * dim ; k < dim ; k++)
    pt[k] = a * coef[id+k] ;

    while (--degre >= 1)
    {
    id -= dim ;
    a = degre ;
    for (k = 0 ; k < dim ; k++)
        pt[k] = a * coef[id+k] + u * pt[k] ;
    }
} /**/

/*------------------------------------------------------------------------*/
void d2Mdu_trajpol (double *pt, double u, double *coef, int dim, int degre)
/*------------------------------------------------------------------------*/
/*
Derivee parametrique 2eme d'une courbe polynomiale
--------------------------------------------------
    Entree :
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome

    Sortie :
    pt      : derivee 2eme calculee
*/
{
int k, id ;
double a ;

    if (degre < 2)
    { for (k = 0 ; k < dim; k++) pt[k] = 0. ; return ; }

    a = degre * (degre-1) ;
    for (k = 0, id = degre*dim ; k < dim ; k++) pt[k] = a * coef[id+k] ;

    while (--degre >= 2)
    {
    id -= dim ;
    a = degre * (degre-1) ;
    for (k = 0 ; k < dim ; k++) pt[k] = a * coef[id+k] + u * pt[k] ;
    }
} /**/

/*------------------------------------------------------------------------*/
void d3Mdu_trajpol (double *pt, double u, double *coef, int dim, int degre)
/*------------------------------------------------------------------------*/
/*
Derivee parametrique 3eme d'une courbe polynomiale
--------------------------------------------------
    Entree :
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome

    Sortie :
    pt      : derivee 3eme calculee
*/
{
int k, id ;
double a ;

    if (degre < 3)
    { for (k = 0 ; k < dim; k++) pt[k] = 0. ; return ; }

    a = degre * (degre-1)*(degre-2) ;
    for (k = 0, id = degre*dim ; k < dim ; k++) pt[k] = a * coef[id+k] ;

    while (--degre >= 3)
    {
        id -= dim ;
        a = degre * (degre-1)*(degre-2) ;
        for (k = 0 ; k < dim ; k++) pt[k] = a * coef[id+k] + u * pt[k] ;
    }
}

/*------------------------------------------------------------------------*/
void d1Mdsu_trajpol(double *dM_ds ,double u, double *coef, int dim, int degre)
/*------------------------------------------------------------------------*/
/*
    Derivee premiere curviligne en u
    --------------------------------
    Entree :
    u       : valeur du parametre
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome

    Sortie :
    dM_ds   : vecteur unitaire tangentiel calcule
*/
{
int k ;
double d12 ;

    d1Mdu_trajpol (dM_ds, u, coef, dim, degre) ;
    d12 = 0.  ; for (k = 0 ; k < dim ; k++) d12 += dM_ds[k] * dM_ds[k] ;
    d12 = sqrt (d12) ;
    if (d12 != 0.) for (k = 0 ; k < dim ; k++) dM_ds[k] /= d12 ;
    else dM_ds[0] = 1. ;
} /**/

/*------------------------------------------------------------------------*/
void d2Mdsu_trajpol(double *d2M_ds2, double u, double *coef, int dim, int degre)
/*------------------------------------------------------------------------*/
/*
    Vecteur courbure en u d'une courbe polynomiale
    ----------------------------------------------
    Entree :
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome

    Sortie :
    d2M_ds2 : vecteur courbure calcule
*/
{
int k ;
double d12, d1d2, a ;
double d1[DIMMAX] , *d2 = d2M_ds2 ;

/* Calcul dM/du et |dM/du|2 */
    d1Mdu_trajpol (d1, u, coef, dim, degre) ;
    d12 = 0.  ; for (k = 0 ; k < dim ; k++) d12 += d1[k] * d1[k] ;

/* Calcul d2M/du2 et <dM/du | d2M/du2> */
    d2Mdu_trajpol (d2, u, coef, dim, degre) ;
    d1d2 = 0. ; for (k = 0 ; k < dim ; k++) d1d2 += d1[k] * d2[k] ;

/* Calcul de :
                   <dM/du | d2M/du2>
       d2M/ds2 = [ (d2M/du2) - ----------------- (dM/du) ] /  |dM/du|2
                   |dM/du|2
*/
    if(d12 != 0.)
    {
      a = d1d2 / d12 ;
      for (k = 0 ; k < dim ; k++) d2M_ds2[k] = (d2[k] - a * d1[k]) / d12 ;
    }
    else for (k = 0 ; k < dim ; k++) d2M_ds2[k] = 0. ;


} /**/

/*------------------------------------------------------------------------*/
void d3Mdsu_trajpol(double *d3M_ds3, double u, double *coef, int dim, int degre)
/*------------------------------------------------------------------------*/
/*
    Vecteur d3M/ds3 en u d'une courbe polynomiale
    ----------------------------------------------
    Entree :
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome

    Sortie :
    d3M_ds3 : vecteur d3M/ds3 calcule
*/
{
int k ;
double dsdu2, dsdu, d2uds2, d3uds3, d1d2, d1d3, d2d2, dsdu3, dsdu4, dsdu5, dsdu7 ;
double d1[DIMMAX] , d2[DIMMAX], *d3 = d3M_ds3 ;

/* Calcul dM/du  d2M/du2 et d3M/du3 */
    d1Mdu_trajpol (d1, u, coef, dim, degre) ;
    d2Mdu_trajpol (d2, u, coef, dim, degre) ;
    d3Mdu_trajpol (d3, u, coef, dim, degre) ;


    dsdu2 = 0.  ; d1d2 = 0. ; d1d3 = 0. ; d2d2 = 0. ; 
    for (k = 0 ; k < dim ; k++) 
    {
        dsdu2 += d1[k] * d1[k] ;
        d1d2 += d1[k] * d2[k] ;
        d1d3 += d1[k] * d3[k] ;
        d2d2 += d2[k] * d2[k] ;
    }

    if (dsdu2 > 0.)
    {
        dsdu = sqrt(dsdu2);
        dsdu3 = dsdu2*dsdu ;
        dsdu4 = dsdu2*dsdu2 ;
        dsdu5 = dsdu4*dsdu ;
        dsdu7 = dsdu5*dsdu2 ;
        d2uds2 = -d1d2/(dsdu2*dsdu2) ;
        d3uds3 = -(d1d3+d2d2)/dsdu5 + 4.*d1d2*d1d2/dsdu7 ;
        for (k = 0 ; k < dim ; k++) 
            d3[k] = d3[k]/dsdu3 + 3.*d2[k]*d2uds2/dsdu + d1[k]*d3uds3 ;
    }
    else for (k = 0 ; k < dim ; k++) d3[k] = 0. ;
} /**/


#if 0
/*------------------------------------------------------------------------*/
double u2s_trajpol (double uf, double *coef, int dim, int degre, double du)
/*------------------------------------------------------------------------*/
/*
    Longueur d'un arc de courbe polynomiale
    ---------------------------------------
    (par sommation des cordes)

    Entree :
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    uf      : Valeur finale de l'intervalle d'integration
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome
    du      : pas d'integration

    Sortie :
    Longueur de la courbe
*/
{
int i, k ;
double s, ds2, u, dm, dut ;
double pti[DIMMAX], ptf[DIMMAX] ;
int nbpas ;

    dut =  uf ;
    if (dut <= 0. || du <= 0.) return 0. ;
    ptu_trajpol (pti, 0., coef, dim, degre) ;
    nbpas = dut / du + 0.5 ;
    if (nbpas == 0) nbpas = 1 ;
    du = dut / nbpas ;
    for (s = 0. , u = du , i = 0 ; i < nbpas ; i++ , u += du)
    {
    ptu_trajpol (ptf, u, coef, dim, degre) ;
    for (ds2 = 0. , k = 0 ; k < dim ; k++)
    {
        dm = ptf[k] - pti[k] ;
        ds2 += dm * dm ;
        pti[k] = ptf[k] ;
    }
    s += sqrt(ds2) ;
    }
    return s ;
}
#endif /**/
#if 1
/*------------------------------------------------------------------------*/
double u2s_trajpol(double uf, double *coef, int dim, int degre, int niss)
/*------------------------------------------------------------------------*/
/*
    Longueur d'un arc de courbe polynomiale
    ---------------------------------------
    (par sommation des derivees)

    Entree :
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    uf      : Valeur finale de l'intervalle d'integration
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome
    niss    : nombre de pas d'integration

    Sortie :
    Longueur de la courbe
*/
{
int n ;
double s = 0., du = uf/niss ;

    for (n = 0 ; n < niss; n++)
    s += dsdu_trajpol ((n+0.5)*du, coef, dim, degre) ;
    s *= du ;
    return s ;
}
#endif /**/

#if 0
/*------------------------------------------------------------------------*/
double longarc_x(double ud, double *coef, double uf, int dim, int degre,
                                  double du)
/*------------------------------------------------------------------------*/
/*
    Longueur d'un arc de courbe polynomiale
    ---------------------------------------
    (par sommation avec correction circulaire)

    Entree :
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    ud      : Valeur initiale de l'intervalle d'integration
    uf      : Valeur finale de l'intervalle d'integration
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome
    du      : pas d'integration

    Sortie :
    Longueur de la courbe
*/
{
int i, k, nbpas ;
double s, ds2, u, dut, dm, ps, v2 ;
double pti[DIMMAX] , ptf[DIMMAX], drv[DIMMAX] ;

    dut = uf - ud ;
    if (dut <= 0. || du <= 0.) return 0. ;
    nbpas = dut / du + 0.5 ;
    if (nbpas == 0) nbpas = 1 ;
    du = dut / nbpas ;

    ptu_trajpol (pti, ud, coef, dim, degre) ;
    for (s = 0. , u = ud+du , i = 0 ; i < nbpas ; i++ , u += du)
    {
    ptu_trajpol (ptf, u, coef, dim, degre) ;
    d1Mdu_trajpol (drv, u, coef, dim, degre) ;
    for (ds2 = 0. , ps = 0., v2 = 0., k = 0 ; k < dim ; k++)
    {
        dm = ptf[k] - pti[k] ;
        ds2 += dm * dm ;
        ps += dm * drv[k] ;
        v2 += drv[k] * drv[k] ;
        pti[k] = ptf[k] ;
    }
    s += ( 4. * sqrt(ds2) - ps / sqrt (v2) ) / 3.  ;
    }
    return s ;
} /**/
#endif
/*------------------------------------------------------------------------*/
double d2Mds_2_MC (double *vec, int dim)
/*------------------------------------------------------------------------*/
/*
    Position du centre de courbure par rapport au point courant
    -----------------------------------------------------------
    Entrees :
    vec     : vecteur courbure
    dim     : dimension espace

    Sorties :
    Carre du rayon et
    vec     : position centre
*/
{
double r2 ;
int k ;
    for (r2 = 0., k = 0 ; k < dim ; k++) r2 += vec[k] * vec[k] ;
    if (r2 == 0.) return r2 ;
    while(dim--) vec[k] /= r2 ;
    return r2 ;
} /**/
/*------------------------------------------------------------------------*/
double dsdu_trajpol (double u, double *coef, int dim, int degre)
/*------------------------------------------------------------------------*/
/*
    Derivee premiere abscisse curviligne / parametrique
    ---------------------------------------------------
    Entree :
    u       : valeur du parametre
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome

    Sortie :
    valeur calculee
*/
{
int k ;
double d12, work[DIMMAX] ;

/* Calcul dM/du et |dM/du|2 */
    d1Mdu_trajpol (work, u, coef, dim, degre) ;
    d12 = 0.  ; for (k = 0 ; k < dim ; k++) d12 += work[k] * work[k] ;
    return sqrt (d12) ;
} /**/
/*------------------------------------------------------------------------*/
double d2sdu_trajpol (double u, double *coef, int dim, int degre)
/*------------------------------------------------------------------------*/
/*
    Derivee seconde abscisse curviligne / parametrique
    --------------------------------------------------
    Entree :
    u       : vaeleur du parametre
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome

    Sortie :
    valeur calculee
*/
{
int k ;
double d12, d1d2 ;
double work1[DIMMAX], work2[DIMMAX] ;

/* Calcul dM/du et |dM/du|2 */
    d1Mdu_trajpol (work1, u, coef, dim, degre) ;
    d2Mdu_trajpol (work2, u, coef, dim, degre) ;
    d12 = 0.  ; d1d2 = 0. ; 
    for (k = 0 ; k < dim ; k++) 
    {
    d12 += work1[k] * work1[k] ;
    d1d2 += work1[k] * work2[k] ;
    }
/*
                     <dM/du | d2M/du2>
       Calcul de :   d2s/du2 =   -----------------
                    |dM/du|
*/
    return d1d2/sqrt(d12) ;
} /**/
/*------------------------------------------------------------------------*/
double d2uds_trajpol (double u, double *coef, int dim, int degre)
/*------------------------------------------------------------------------*/
/*
    Derivee seconde abscisse parametrique / curviligne
    --------------------------------------------------
    Entree :
    u       : vaeleur du parametre
    Coef    : Coefficients des polynomes presentes en vecteur :
          Coef[i*dim+k] est la composante k du coef de degre i
    dim     : dimension de l'espace de la courbe
    degre   : degre du polynome

    Sortie :
    valeur calculee
*/
{
int k ;
double d12, d1d2 ;
double work1[DIMMAX], work2[DIMMAX] ;

/* Calcul dM/du et |dM/du|2 */
    d1Mdu_trajpol (work1, u, coef, dim, degre) ;
    d12 = 0.  ; for (k = 0 ; k < dim ; k++) d12 += work1[k] * work1[k] ;

/* Calcul d2M/du2 et <dM/du | d2M/du2> */
    d2Mdu_trajpol (work2, u, coef, dim, degre) ;
    d1d2 = 0. ; for (k = 0 ; k < dim ; k++) d1d2 += work1[k] * work2[k] ;

/*
                   - <dM/du | d2M/du2>
       Calcul de :   d2u/ds2 =   -----------------
                    |dM/du|4
*/
    return -d1d2/(d12*d12) ;
} /**/
/*------------------------------------------------------------------------*/
void dori_trajpol (double *coef, int dim, int degre, double du)
/*------------------------------------------------------------------------*/
/*
Decalage de l'origine de l'abscisse par. d'une trajectoire polynomiale
----------------------------------------------------------------------
    Entree :
    coef    : Coefficients des polynomes presentes en vecteur :
          coef[i*dim+k] est la composante k du coef de degre i.
    dim     : dimension de l'espace de la courbe.
    degre   : degre du polynome.
    du      : valeur du decalage de l'abscisse parametrique.

    Sortie :
    coef    : Les coefficients sont recalcules de telle maniere que
          M(u+du) = M'(u).
    Formule utilisee :
    Notons Ak le coefficient du terme de degre k. Il vient :
           N     k     (n-k)
        A  = Somme  C  A du
         k   n = k   n  n
*/
{
double fac[DEGMAX+1], dun[DEGMAX+1], ck ;
double *ccf, *ccc ;
int i, j, k ;

    dun[0] = 1. ; fac[0] = 0. ;
    for (i = 1 ; i <= degre ; i++) { fac[i] = 0. ; dun[i] = dun[i-1] * du ; }

    for (i = 0 , ccf = coef ; i < degre ; i++, ccf += dim)
    for (j = 1 , ccc = ccf + dim , ck = 1. ; j <= degre-i ; j++, ccc +=dim)
    {
    ck += fac[j] ;
    for (k = 0 ; k < dim ; k++) ccf[k] += ck * ccc[k]* dun[j] ;
    fac[j] = ck ;
    }
}

/*-----------------------------------------------------------------------*/
double s2u_trajpol (double s, double uf, double *coef,
                                          int dim, int degre, int niss)
/*-----------------------------------------------------------------------*/
/*
        Interpolation abscisse parametrique u = f(s)
    --------------------------------------------
    Entree :
    s       : valeur de l'abscisse curviligne
    coef    : Coefficients des polynomes presentes en vecteur :
              Coef[i*dim+k] est la composante k du coef de degre i
    uf          : valeur finale u
    epsu    : espu d'interpolation ayant servi a l'integration
    dim     : dimension de l'espace de la courbe
        degre   : degre du polynome

        Sortie :
        valeur calculee
*/
{
double epsu = uf/niss, s1, s2, u, du, dus2, ufmu ;

    if (s <= 0.) return 0. ;
    s2 = 0. ; u = 0. ; du = epsu ; dus2 = du/2. ;
/* recherche s1 et s2 tels que s1 < s <= s2 */
    while (s > s2)
    {
    /* Ajustement du = epsu sauf dernier segment */
    ufmu = uf - u ;                /* Distance a la fin */
    if (du > ufmu)                 /* intervalle trop grand ou uc > uf */
    { du = ufmu > 0. ? ufmu : epsu ; dus2 = du/2. ; }
    s1 = s2 ;
    s2 += dsdu_trajpol (u + dus2, coef, dim, degre) * du ;
    u += du ;
    }
    s1 = s2 - s1 ;
    if (s1 != 0.) return u - (s2 - s) *  du / s1 ;
    else return u ;
}

/*-----------------------------------------------------------------------*/
double long_estim (double *p1, double *p2, double *t1, double *t2, int dim)
/*-----------------------------------------------------------------------*/
/*
Estimation de la longueur curviligne d'une trajectoire polynomiale
------------------------------------------------------------------
    Entree :
    p1      : Point depart
    p2      : Point arrivee
    t1      : Tangente depart
    t2      : Tangente arrivee
    dim     : Dimension espace
    Sortie :
    retourne la longueur estimee
*/
/*-----------------------------------------------------------------------*/
{
double d= 0., sa = 0., lt1 = 0., lt2 = 0. ;
int k ;

    for (k = 0; k < dim ; k++)
    {
    d += (p2[k] - p1[k]) * (p2[k] - p1[k]) ;
    lt1 += t1[k]*t1[k] ;
    lt2 += t2[k]*t2[k] ;
    }

    d = sqrt (d) ;
    lt1 = sqrt (lt1) ;
    lt2 = sqrt(lt2) ;
    for (k = 0; k < dim ; k++)
    sa += (p2[k] - p1[k]) * (t1[k]/lt1 + t2[k]/lt2) ;
    sa = 4. * d / 3. - sa / 6. ;
    if (sa < d) sa = d ;
    return sa ;
}
/*----------------------------------------------------------------------------
   PROCEDURES DE CALCUL DES COEFFICIENTS DE RACCORD POLYNOMES
----------------------------------------------------------------------------*/

/*-------------------------------------------------------------*/
/* RACCORD 2 points par une droite                             */
/*-------------------------------------------------------------*/

void trajpol_drte00 (double *A1, double Uf, double *A2, int dim)
{
int i ;
double *B1 = A1 + dim ;
    for (i = 0 ; i < dim ; i++)
    B1[i] = (A2[i] - A1[i])/Uf ;
}


/*-------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une cubique (b1 et b2 donnes) */
/*-------------------------------------------------------------*/
void trajpol_cubi11(double *A1, double d, double *A2, int n)
/*--------------------------------------------------------------------------*/
/*
    Calcul du polynome du 3eme degre qui passe par les points A1 et A2,
    avec les tangentes B1 et B2 donnees.

    On a M(u) = A1 + B1 u + C1 u2 + D1 u3

    La procedure calcule les coefficients C1 et D1 :
    On pose :
        S = (A2 - A1) / d
    Il vient :
        C1 = (3S -B2 - 2B1) / d
        D1 = (B1 + B2 - 2S) / (d*d)
*/
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n, *D1 = C1 + n ;
double *B2 = A2 + n ;
double d2, s ;

    d2 = d * d ;

    while (n--)
    {
        s = ( A2[n] - A1[n] ) / d ;
        C1[n] = (3. * s - B2[n] - 2.* B1[n]) / d ;
        D1[n] = (B1[n] + B2[n] - 2. * s) / d2 ;
    }
}
/*------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par un polfive (b1,c1 et b2,c2 donnes) */
/*------------------------------------------------------------------*/
void trajpol_five22(double *A1, double d, double *A2, int n)
/*--------------------------------------------------------------------------*/
/*
    Calcul du polynome du 5eme degre qui passe par les points A1 et A2,
    avec les tangentes B1 et B2 et les demi-courbures C1 et C2 donnees.

    On a M(u) = A1 + B1 u + C1 u2 + D1 u3 + E1 u4 + F1 u5

    La procedure calcule les coefficients D1, E1 et F1 :
        On pose :
                S1 = (A2 - A1) / d1
                G1 = (S1 - B1) / d1
                H1 = (B2 - B1) / d1
        Il vient :
                D1 = (C2 - 3 C1 - 4 H1 + 10 G1 ) / d1
                E1 = (3 C1 - 2 C2 + 7 H1 - 15 G1 ) / (d1*d1)
                F1 = (C2 - C1 - 3 H1 + 6 G1 ) / (d1*d1*d1)
*/
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n, *D1 = C1 + n, *E1 = D1 + n, *F1 = E1 + n ;
double *B2 = A2 + n , *C2 = B2 + n ;
double d2, d3, d4, t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, g, h ;

    d2 = d * d   ; d3  = d2 * d  ; d4  = d3 * d  ;
    t1 = 1. / d  ; t2  = 3. / d  ; t3  = 4. / d2 ; t4 = 10. / d2 ;
    t5 = 3. / d2 ; t6  = 2. / d2 ; t7  = 7. / d3 ; t8 = 15. / d3 ;
    t9 = 1. / d3 ; t10 = 3. / d4 ; t11 = 6. / d4 ;

    while (n--)
    {
        g = ( ( A2[n] - A1[n] ) / d  - B1[n] ) ;
        h = ( B2[n] - B1[n] ) ;
        D1[n] = t1 * C2[n] - t2 * C1[n] - t3 * h + t4 * g ;
        E1[n] = t5 * C1[n] - t6 * C2[n] + t7 * h - t8 * g ;
        F1[n] = t9 * (C2[n] - C1[n])   - t10 * h + t11 * g ;
    }
}
/*-------------------------------------------*/
/* RACCORD c2 entre 3 points par 2 paraboles */
/*-------------------------------------------*/
void trajpol_2boles00(double *A1, double d1, double *A2, double d2,
                                                        double *A3, int n)
/*--------------------------------------------------------------------------*/
/*
    Calcul des deux paraboles qui passent par les points A1, A2 et A3,

    On a M1(u) = A1 + B1 u + C1 u2 et M2(u) = A2 + B2 u + C2 u2

    On pose :
                S1 = (A2 - A1) / d1
                S2 = (A3 - A2) / d2
    La procedure calcule les coefficients B1, C1=C2 et B2 :
                C1 = C2 = (S2 - S1) / (d1 + d2)
                B1 = [S1*(2d1+d2) - d1*S2] / (d1 + d2)
                B2 = [S1*d2 + d1*S2] / (d1 + d2)
*/
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n ;
double *B2 = A2 + n , *C2 = B2 + n ;
double s1, s2, ss = d1 + d2 ;
    while (n--)
    {
        s1 = (A2[n] - A1[n]) / d1 ;
        s2 = (A3[n] - A2[n]) / d2 ;
        C1[n] = C2[n] = (s2 - s1)/ss ;
        B1[n] = s1 - C1[n] * d1 ;
        B2[n] = s2 - C2[n] * d2 ;
    }
}
/*------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une quadrique (b1,c1 et b2 donnes) */
/*------------------------------------------------------------------*/
void trajpol_quad21(double *A1, double d, double *A2, int n)
/*--------------------------------------------------------------------------*/
/*
    Calcul du polynome du 4eme degre qui passe par les points A1 et A2,
    avec les tangentes B1 et B2 et la demi-courbure C1 donnees.

    On a M(u) = A1 + B1 u + C1 u2 + D1 u3 + E1 u4

    La procedure calcule les coefficients D1 et E1 :
        On pose :
                S1 = (A2 - A1) / d1
                G1 = (S1 - B1) / d1
                H1 = (B2 - B1) / d1
        Il vient :
                D1 = (- H1 - 2 C1 + 4 G1 ) / d1
                E1 = (  H1 +   C1 - 3 G1 ) / (d1*d1)
*/
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n, *D1 = C1 + n, *E1 = D1 + n ;
double *B2 = A2 + n ;
double d2, g, h ;

    d2 = d * d   ;

    while (n--)
    {
        g = ( ( A2[n] - A1[n] ) / d  - B1[n] ) / d ;
        h = ( B2[n] - B1[n] ) / d ;
        D1[n] = ( - h - 2. * C1[n] + 4. * g) / d  ;
        E1[n] = (   h +      C1[n] - 3. * g) / d2 ;
    }
}
/*------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une quadrique (b1 et b2,c2 donnes) */
/*------------------------------------------------------------------*/
void trajpol_quad12(double *A1, double d, double *A2, int n)
/*--------------------------------------------------------------------------*/
/*
    Calcul du polynome du 4eme degre qui passe par les points A1 et A2,
    avec les tangentes B1 et B2 et la demi-courbure C2 donnees.

    On a M(u) = A1 + B1 u + C1 u2 + D1 u3 + E1 u4

    La procedure calcule les coefficients C1, D1 et E1 :
        On pose :
                S1 = (A2 - A1) / d1
                G1 = (S1 - B1) / d1
                H1 = (B2 - B1) / d1
        Il vient :
                C1 = (  6 G1 - 3 H1 +   C2)
                D1 = (- 8 G1 + 5 H1 - 2 C2) / d1
                E1 = (  3 G1 - 2 H1 +   C2) / (d1*d1)
*/
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n, *D1 = C1 + n, *E1 = D1 + n ;
double *B2 = A2 + n , *C2 = B2 + n ;
double d2, g, h ;

    d2 = d * d   ;

    while (n--)
    {
        g = ( ( A2[n] - A1[n] ) / d  - B1[n] ) / d ;
        h = ( B2[n] - B1[n] ) / d ;
        C1[n] = ( 6. * g - 3. * h +      C2[n]) ;
        D1[n] = (-8. * g + 5. * h - 2. * C2[n]) / d  ;
        E1[n] = ( 3. * g - 2. * h +      C2[n]) / d2 ;
    }
}
/*----------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une cubique (b1,c1 donnes) */
/*----------------------------------------------------------*/
void trajpol_cubi20(double *A1, double d1, double *A2, int n)
/*--------------------------------------------------------------------------*/
/*
    Calcul du polynome du 3eme degre qui passe par les points A1 et A2,
    avec la tangente B1 et la demi-courbure C1 donnees.

    On a M(u) = A1 + B1 u + C1 u2 + D1 u3

    La procedure calcule le coefficient D1 :
                D1 = (((A2 - A1)/d1 - B1)/d1 - C1)/d1
*/
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n, *D1 = C1 + n ;
    while (n--)
        D1[n] = (((A2[n]-A1[n])/d1 -  B1[n])/d1 - C1[n])/d1 ;
}
/*-------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une parabole (b1 donne) */
/*-------------------------------------------------------*/
void trajpol_bole10(double *A1, double d1, double *A2, int n)
/*--------------------------------------------------------------------------*/
/*
    Calcul de la parabole qui passe par les points A1 et A2,
    avec la tangente B1 donnee.

    On a M(u) = A1 + B1 u + C1 u2

    La procedure calcule le coefficient C1 :
                C1 = (((A2 - A1)/d1 - B1)/d1
*/
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n ;
    while (n--)
        C1[n] = ((A2[n]-A1[n])/d1-B1[n])/d1 ;
}
/*-------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une parabole (b2 donne) */
/*-------------------------------------------------------*/
void trajpol_bole01(double *A1, double d1, double *A2, int n)
/*--------------------------------------------------------------------------*/
/*
    Calcul de la parabole qui passe par les points A1 et A2,
    avec la tangente B2 donnee.

    On a M(u) = A1 + B1 u + C1 u2

    On pose S = (A2-A1)/d1
    La procedure calcule le coefficient C1 :
                B1 = 2S - B2
                C1 = (B2 - S)/d1
*/
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n , *B2 = A2 + n ;
double s, g ;
    while (n--)
    {
        s = (A2[n] - A1[n])/d1 ;
        g = B2[n] - s ;
        B1[n] = s - g ;
        C1[n] = g/d1 ;
    }
}
/*----------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une cubique (b2,c2 donnes) */
/*--------------------------------------------------------------------------*/
void trajpol_cubi02(double *A1, double d1, double *A2, int n)
/*--------------------------------------------------------------------------*/
/*
    Calcul du polynome du 3eme degre qui passe par les points A1 et A2,
    avec la tangente B2 et la demi-courbure C2 donnees.

    On a M(u) = A1 + B1 u + C1 u2 + D1 u3

    On pose :
                S1 = (A2 - A1) / d1
    La procedure calcule les coefficients B1, C1 et D1 :
                B1 = 3*S1 - 2*B2 + C2*d1
                C1 = (-3*S1 + 3*B2)/d1 - 2*C2
                D1 = ((S1 - B2)/d1 + C2)/d1
*/
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n, *D1 = C1 + n ;
double *B2 = A2 + n , *C2 = B2 + n ;
double s, g ;
    while (n--)
    {
        s = (A2[n] - A1[n]) / d1 ;
        g = ( s - B2[n] ) / d1 ;
        B1[n] = 3.* s -  2. * B2[n] + C2[n]*d1 ;
        C1[n] = -3.*g - 2.*C2[n] ;
        D1[n] = (g + C2[n])/d1 ;
    }
}
/*--------------------------------------------------------------------*/
/* RACCORD c2 entre 3 points par 2 quadriques (b1,c1 et b3,c3 donnes) */
/*--------------------------------------------------------------------*/
void trajpol_2quads22 (double *A1, double d1, double *A2, double d2,
                                                        double *A3, int n)
/*--------------------------------------------------------------------------*/
/*
   Calcul des 2 polynomes du 4eme degre qui passe par les points A1, A2 et A3
   avec les tangentes B1 et B3 et les demi-courbures C1 et C3 donnees.

    On a
        M1(u) = A1 + B1 u + C1 u2 + D1 u3 + E1 u4
        M2(u) = A2 + B2 u + C2 u2 + D2 u3 + E2 u4

    La procedure calcule les coefficients D1, E1, B2, C2, D2 et E2 :
        On pose :

              S1 = (A2 - A1) / d1
              S2 = (A3 - A2) / d2

        Il vient :


        B2 = (d1 d2 (C3 - C1)/3 + d2 (2 S1 + B1) + d1 (2 S2 + B3)) / (d1 + d2)
        C2 = (-6 S1 + 3 B1 + d1 C1 + 6 S2 - 3 B3 + d2 C3) / (d1 + d2)

        D1 = (4 S1 - 3 B1 - 2 d1 C1 - B2) / (d1 * d1)
        E1 = (-3 S1 + 2 B1 + d1 C1 + B2) / (d1 * d1 * d1)

        D2 = (4 S2 - 3 B2 - 2 d2 C2 - B3) / (d2 * d2)
        E2 = (-3 S2 + 2 B2 + d2 C2 + B3) / (d2 * d2 * d2)
*/
/*--------------------------------------------------------------------------*/
{
double *B1 = A1 + n , *C1 = B1 + n, *D1 = C1 + n, *E1 = D1 + n ;
double *B2 = A2 + n , *C2 = B2 + n, *D2 = C2 + n, *E2 = D2 + n ;
double *B3 = A3 + n , *C3 = B3 + n ;
double dd1, dd2, d12, d22, d13, d23, d1d2s3, d1pd2, s1, s2 ;

    dd1 = 2. * d1 ;
    dd2 = 2. * d2 ;
    d12 = d1 * d1 ;
    d22 = d2 * d2 ;
    d13 = d12 * d1 ;
    d23 = d22 * d2 ;
    d1d2s3 = d1 * d2 / 3. ;
    d1pd2 = d1 + d2 ;
    while (n--)
    {
        s1 = (A2[n] - A1[n]) / d1 ;
        s2 = (A3[n] - A2[n]) / d2 ;

        B2[n] = (d1d2s3 * (C3[n] - C1[n]) + d2 * (2. * s1 - B1[n])
                                          + d1 * (2. * s2 - B3[n]) ) / d1pd2 ;
        C2[n] = (3. * (B1[n] - B3[n]) +  6. * ( s2 - s1)
                            + d1 * C1[n] + d2 * C3[n]) / d1pd2 ;

        D1[n] = (4. * s1 - 3. * B1[n] - dd1 * C1[n] - B2[n]) / d12 ;
        D2[n] = (4. * s2 - 3. * B2[n] - dd2 * C2[n] - B3[n]) / d22 ;

        E1[n] = (2. * B1[n] - 3. * s1 + d1 * C1[n] + B2[n]) / d13 ;
        E2[n] = (2. * B2[n] - 3. * s2 + d2 * C2[n] + B3[n]) / d23 ;

    }
}

/*--------------------------------------------------------------------------*/
void trisplin (double **A, double *d, int typdeb, int typfin, int NBSG,
                                  int NBCOMP)
/*--------------------------------------------------------------------------*/
/*
    ENTREES : A, d, typdeb, typfin, NBSG, NBCOMP
    A est un pointeur sur NBSG+1 blocs de doubles.
        Dans chaque bloc il y au moins la place pour 4 coefficients
        avec NBCOMP composantes, sauf le dernier (NBSG+1) qui peut ne
        contenir la place que pour 3 coefficients.
    d est le tableau des valeurs finales des paramtres des NBSG segments

    typdeb decrit le type de raccord en debut de trajectoire :
            NEANT ou
            NDFIC  : rien specifie
            CRNUL  : pas de tangente et courbure nulle au depart
            TGTE   : tangente donnee
            QUADRI : raccord par quadrique
        typfin decrit le type de raccord en fin de trajectoire (idem typdeb)
    SORTIES : B, C, D calculees dans les tableaux pointes par A
*/
/*--------------------------------------------------------------------------*/
{
int n1, nf, nfp1 ;
int i, ix1, j ;
double **B, **C, **D, **H, **S ;
double *a, *b, *c, g, g2, g3 ;

/* Allocation pour tableaux de coefficients internes a, b et c
       et pour tableaux de pointeurs sur coefs B=H, C, et D=S */
    a = malloc (6*(NBSG+1)*sizeof (double) ) ;
    if (a == NULL)
    { puts ("Echec allocation dans trisplin !") ; exit (1) ; }
    b = &a[NBSG+1] ; c = &b[NBSG+1] ;

    H = B = (double **) &c[NBSG+1] ;
    C = &B[NBSG+1] ;
    S = D = &C[NBSG+1] ;

/* Initialisation des pointeurs sur les coefs B, C et D */
    for (i = 0 ; i <= NBSG ; i++)
    {
        B[i] = A[i] + NBCOMP ;
        C[i] = B[i] + NBCOMP ;
        D[i] = C[i] + NBCOMP ;
    }

    n1 = 0 ;            /* index 1er segment de la spline "cubique" */
    nf = NBSG - 1 ;     /* index dernier segment de la spline "cubique" */
    if (typdeb == QUADRI) n1++ ;
    if (typfin == QUADRI) nf-- ;
    nfp1 = nf+1 ;
    if (nf-n1 < 1)
    {
       if (typdeb == NDFIC) typdeb = CRNUL ;
       if (typfin == NDFIC) typfin = CRNUL ;
    }

/* Calcul S1 */
    if (d[n1] != 0.)
        for (j = 0 ; j < NBCOMP ; j++)
        S[n1][j] = (A[n1+1][j] - A[n1][j]) / d[n1] ;
    else for (j = 0 ; j < NBCOMP ; j++) S[n1][j] = 0. ;

/* Initialisation b1, c1, H1 */
    switch (typdeb) {
    case TGTE :
        b[0] = 1. ;
        c[0] = 0. ;
        /* H1 deja en place par partage memoire de B1 */
        break ;
    case CRNUL :
        b[0] = 2. ;
        c[0] = 1. ;
        for (j = 0 ; j < NBCOMP ; j++)
            H[0][j] = 3.* S[0][j] ;
        break ;
    case NDFIC :
    b[0] = d[1] ;
    c[0] = d[0] + d[1] ;
    g = d[1] * (3. * d[0] + 2. * d[1] ) ;
    if (d[1] != 0.) g2 = d[0] * d[0] / d[1] ;
        else g2 = 0. ;
    g3 = d[0] + d[1] ;
        if (g3 != 0.)
           for (j = 0 ; j < NBCOMP ; j++)
              H[0][j] = ( g * S[0][j] + g2 * (A[2][j] - A[1][j]) ) / g3 ;
        else for (j = 0 ; j < NBCOMP ; j++) H[0][j] = 0. ;
        break ;
    case QUADRI :
        /* Calcul S0  */
    if (d[0] != 0.)
          for (j = 0 ; j < NBCOMP ; j++)
        S[0][j] = (A[1][j] - A[0][j]) / d[0] ;
        else for (j = 0 ; j < NBCOMP ; j++) S[0][j] = 0. ;

    b[1] = 2.* d[0] + 3.* d[1] ;
    c[1] = d[0] ;
        for (j = 0 ; j < NBCOMP ; j++)
        H[1][j] = 3.* d[0] * S[1][j]
            + 6. * d[1] * S[0][j]
            - 3. * d[1] * B[0][j] - d[0] * d[1] * C[0][j] ;
        break ;
    }

/* Calcul ai, bi, ci, Si, Hi pour n1+1 a nf */
    ix1 = n1  ;
    for (i = n1+1 ; i <= nf ; i++)
    {
    a[i] = d[i] ;
    b[i] = 2. * (d[ix1] + d[i]) ;
    c[i] = d[ix1] ;
    if (d[i] != 0.)
            for (j = 0 ; j < NBCOMP ; j++)
            {
        S[i][j] = (A[i+1][j] - A[i][j]) / d[i] ;
        H[i][j] = 3. * ( d[i] * S[ix1][j] + d[ix1] * S[i][j] ) ;
            }
        else
            for (j = 0 ; j < NBCOMP ; j++)
            { S[i][j] = 0. ; H[i][j] = 0. ; }
        ix1 = i ;
    }

/* Calcul an+1, bn+1, Hn+1 */
    switch (typfin) {
    case TGTE :
        a[nfp1] = 0. ;
        b[nfp1] = 1. ;
        /* Hn+1 deja en place par partage memoire de Bn+1 */
        break ;
    case CRNUL :
        a[nfp1] = 1. ;
        b[nfp1] = 2. ;
        for (j = 0 ; j < NBCOMP ; j++)
            H[nfp1][j] = 3.* S[nf][j] ;
        break ;
    case NDFIC :
    a[nfp1] = d[nf-1] + d[nf] ;
    b[nfp1] = d[nf-1] ;
    g = d[nf-1] * (3. * d[nf] + 2. * d[nf-1] ) ;
    g2 = d[nf] * d[nf] ;
    g3 = d[nf-1] + d[nf] ;
        if (g3 != 0.)
           for (j = 0 ; j < NBCOMP ; j++)
              H[nfp1][j] = ( g * S[nf][j] + g2 * S[nf-1][j] ) / g3 ;
        else
           for (j = 0 ; j < NBCOMP ; j++) H[nfp1][j] = 0. ;
        break ;
    case QUADRI :
        /* Calcul Sn+1  */
    if (d[nfp1] != 0.)
          for (j = 0 ; j < NBCOMP ; j++)
        S[nfp1][j] = (A[nfp1+1][j] - A[nfp1][j]) / d[nfp1] ;
        else for (j = 0 ; j < NBCOMP ; j++) S[nfp1][j] = 0. ;

    a[nfp1] = d[nfp1] ;
    b[nfp1] = 3.*d[nf] + 2.*d[nfp1] ;
        for (j = 0 ; j < NBCOMP ; j++)
        H[nfp1][j] = 3.*d[nfp1]*S[nf][j] + d[nf]*(6.*S[nfp1][j]
               - 3.*B[nfp1+1][j] + d[nfp1]*C[nfp1+1][j]) ;
        break ;
    }

/* Rsolution systme tridiagonal */
    for (i = n1 + 1 ; i <= nfp1 ; i++)
    {
        if (b[i-1] != 0.) g = a[i]/b[i-1] ;
        else g = 0. ;
        b[i] -= g * c[i-1] ;
        for (j = 0 ; j < NBCOMP ; j++)
            H[i][j] -= g * H[i-1][j] ;
    }

    if (b[nfp1] != 0.)
        for (j = 0 ; j < NBCOMP ; j++)
            B[nfp1][j] = H[nfp1][j]/b[nfp1] ;
    else for (j = 0 ; j < NBCOMP ; j++)
            B[nfp1][j] = 0. ;

    ix1 = nfp1 ;
    for (i = nf ; i >= n1 ; i--)
    {
        if (b[i] != 0.)
            for (j = 0 ; j < NBCOMP ; j++)
                B[i][j] = (H[i][j] - c[i]*B[ix1][j])/b[i] ;
        else for (j = 0 ; j < NBCOMP ; j++)
                B[i][j] = 0. ;
        ix1 = i ;
    }

/* Rsolution quations (5) et (6) */
   for (i = n1 ; i <= nf ; i++)
   {
       ix1 = i + 1 ;
       if (d[i] != 0.)
           for (j = 0 ; j < NBCOMP ; j++)
           {
           C[i][j] = (3. * S[i][j] - 2. * B[i][j] - B[ix1][j]) / d[i] ;
           D[i][j] = (B[i][j] + B[ix1][j] - 2. * S[i][j])/(d[i] * d[i]) ;
           }
       else for (j = 0 ; j < NBCOMP ; j++)
           {   C[i][j] = 0. ; D[i][j] = 0. ; }
   }

   if (typdeb == QUADRI)
   {
/* Quadrique initiale */
    g = 2. * d[0] ;
    g2 = d[0] * d[0] ;
    g3 = g2 * d[0] ;
        if (g != 0.) for (j = 0 ; j < NBCOMP ; j++)
        {
            D[0][j+NBCOMP] =
        (-3.*S[0][j] + C[0][j]* d[0] + 2.*B[0][j] + B[1][j]) / g3 ;
                /* Respecter l'ordre car D ecrase S */
            D[0][j] =
                (4. * S[0][j] - g * C[0][j] - 3. * B[0][j] - B[1][j]) / g2 ;
        }
        else for (j = 0 ; j < NBCOMP ; j++)
        {   D[0][j] = 0. ; D[0][j+NBCOMP] = 0. ; }
    }

   if (typfin == QUADRI)
   {
/* Quadrique finale */
    g = 2. * d[nfp1] ;
    g2 = d[nfp1] * d[nfp1] ;
    g3 = g2 * d[nfp1] ;
    if (d[nfp1] != 0.) for (j = 0 ; j < NBCOMP ; j++)
        {
            C[nfp1][j] = C[nfp1+1][j] +
        (6. * S[nfp1][j] - 3.* (B[nfp1+1][j] + B[nfp1][j])) / d[nfp1] ;
            D[nfp1][j+NBCOMP] =
        (3.*S[nfp1][j] + C[nfp1+1][j]* d[nfp1] - 2. * B[nfp1+1][j]
                          - B[nfp1][j]) / g3 ;
            D[nfp1][j] =
                (-8. * S[nfp1][j] - g * C[nfp1+1][j] + 5. * B[nfp1+1][j]
                     + 3. * B[nfp1][j]) / g2 ;
        }
        else for (j = 0 ; j < NBCOMP ; j++)
        {   C[nfp1][j] = C[nfp1+1][j] ;
                        D[nfp1][j] = D[nfp1][j+NBCOMP] = 0. ; }
    }

   free (a) ;
}

/*--------------------------------------------------------------------------*/
void lispline (double **coefs, double *uf, int typdeb, int typfin, int nseg,
                          int dim, double *dy, double la)
/*--------------------------------------------------------------------------*/
/*
    ENTREES : idem trisplin : coefs, uf, typdeb, typfin, nseg, dim
    coefs est un pointeur sur NBSG+1 blocs de doubles.
          Dans chaque bloc il y au moins la place pour 4 coefficients
          avec NBCOMP composantes, sauf le dernier (NBSG+1) qui peut ne
          contenir la place que pour 3 coefficients.
    uf    est le tableau des valeurs finales
          des paramtres des NBSG+1 segments (dernier bidon mais utilise).
    typdeb decrit le type de raccord en debut de trajectoire :
          NEANT ou
          NDFIC  : rien specifie
          CRNUL  : pas de tangente et courbure nulle au depart
          TGTE   : tangente donnee
          QUADRI : raccord par quadrique
    typfin decrit le type de raccord en fin de trajectoire (idem typdeb)
    nseg  est le nombre de segments
    dim   est la dimension de l'espace ponctuel de la trajectoire
    En plus de trisplin :
    dy    est le tableau des tolerances aux nseg+1 points
    la    est le coefficient de filtrage compris entre 0. et 1.
          0. : pas de filtrage : passage exact par les points
          1.  : filtrage maximal
Remarque :
    A l'origine les points M sont en place dans les tableaux pointes par
    coefs. Avant calcul des coordonnes des points de passage effectifs,
    qui constituent les coefficients des polynomes, on transfere les
    valeurs des anciennes coordonnees a l'emplacement des coefficients
    de degre 5 (non utilises) des polynomes dans les tableaux pointes par
    coefs.

    SORTIES : Coefficients polynomiaux dans les tableaux pointes par coefs
*/
/*--------------------------------------------------------------------------*/
{
int i, n, ideb, ifin, npt, nsm1, ida, ifa ;
double d, d2, temp, bid, six1mla, deuxla, troisla ;
double *qtd_d, *qtd_s, *qtd_i, *v1, *v2, *v3, *qtm, *ui ;
double **A, **B, **C, **D, **E;

    if (la < 0.) la = 0. ;
    if (la > 1.) la = 1. ;
    la = pow(la,0.1) ;
    if (la < 0.) la = 0. ;
    if (la > 1. -1.e-10) la = 1. -1.e-10 ;

    la = 1.- la ;   /* Definition inverse du rapport */
    six1mla = 6.* (1.-la) ;
    deuxla = 2. * la ;
    troisla = 3.* la ;
    npt = nseg + 1 ;
    nsm1 = nseg - 1 ;
/* Allocation pour les trois bandes de QT*D  et de QT*D*D*Q, QT*Y et U */
    qtd_d = (double *) malloc ((13*npt+1)*sizeof(double)) ;
    if (qtd_d == NULL)
    {
    puts ("Echec allocation memoire dans Lispline.c !!") ;
    exit (1) ;
    }
    qtd_i = qtd_d + npt ;
    qtd_s = qtd_i + npt ;
    v1 = qtd_s + npt ; v2 = v1 + npt + 1 ; v3 = v2 + npt ;
    qtm = v3 + npt ; ui = qtm + npt ;
    A = (double **) (ui + npt) ;
    B = A + npt ; C = B + npt ; D = C + npt ; E = D + npt ;

    for (n = 0 ; n < npt ; n++)
    {
    A[n] = coefs[n] ;
    B[n] = A[n] + dim ;
    C[n] = B[n] + dim ;
    D[n] = C[n] + dim ;
    E[n] = D[n] + dim ;
    }

/*--------------------------------------------------------------------------*/
/*                          Calcul de QT*D                                  */
/*  La matrice qtd est constituee d'une diagonale qtd_d, d'une diagonale    */
/*  superieure qtd_s et d'une diagonale inferieure qtd_i.                   */
/*  qtd_d a nseg+1 elements indices de 0 a nseg                             */
/*  qtd_s a nseg elements indices de 0 a nseg-1 et l'element fictif nseg    */
/*  qtd_i a nseg elements indices de 1 a nseg et l'element fictif 0         */
/*  Les elements fictifs permettent d'effectuer les operations ulterieures  */
/*  avec moins de test sur les bornes des indices.                          */
/*--------------------------------------------------------------------------*/
    qtd_i[0] = 0. ;
    if (typdeb == 1) { qtd_d[0] = - dy[0]/ uf[0] ; qtd_s[0] =   dy[1]/ uf[0] ; }
    else { qtd_d[0] = qtd_s[0] = 0. ; }
    for (n = 1 ; n < nseg ; n++)
    {
    qtd_i[n] = dy[n-1]/ uf[n-1] ;
    qtd_d[n] = - dy[n]/ uf[n-1] - dy[n]/ uf[n] ;
    qtd_s[n] = dy[n+1]/ uf[n] ;
    }
    if (typdeb == 2) {qtd_i[1] = 0. ; qtd_d[1] -= dy[1]/ uf[0] ; }
    if (typfin == 1)
    {
    qtd_i[nseg] = dy[nsm1]/ uf[nsm1] ;
    qtd_d[nseg] = - dy[nseg]/ uf[nsm1] ;
    }
    else { qtd_i[nseg] = qtd_d[nseg] = 0. ; }
    if (typfin == 2) { qtd_d[nsm1] -= dy[nsm1]/ uf[nsm1] ; qtd_s[nsm1] = 0. ; }
    qtd_s[nseg] = 0. ;

/*--------------------------------------------------------------------------*/
/*                         CALCUL DE QT*D*D*Q                               */
/*  La matrice qtddq est symetrique et la partie calculee comprend la       */
/*  diagonale v1, la 1ere surdiagonale qtddq_s1 et la 2eme qtddq_s2         */
/*  v1 a nseg+1 elements indices de 0 a nseg                                */
/*  v2 a nseg elements indices de 0 a nseg-1 (+ i_nseg fictic)              */
/*  v3 a nseg-1 elemenst indices de 0 a nseg-2 (+ nseg-1 et nseg fictifs)   */
/*--------------------------------------------------------------------------*/

/* Calcul diagonale de QT*D*D*Q  */
    for (n = 0 ; n <= nseg ; n++)
    v1[n] = qtd_i[n]*qtd_i[n] + qtd_d[n]*qtd_d[n] + qtd_s[n]*qtd_s[n] ;

/* Calcul premiere surdiag */
    for (n = 0 ; n < nseg ; n++)
    v2[n] = qtd_d[n]*qtd_i[n+1] + qtd_s[n]*qtd_d[n+1] ;
    v2[nseg] = 0. ;

/* Calcul deuxieme surdiag */
    for (n = 0 ; n < nsm1 ; n++) v3[n] = qtd_s[n]*qtd_i[n+2] ;
    v3[nsm1] = v3[nseg] = 0. ;

/*--------------------------------------------------------------------------*/
/*            CONSTRUCTION DE G = 6(1-la)QT*D*D*Q + la*R                    */
/*--------------------------------------------------------------------------*/

/* Calcul diagonale  */
    v1[0] = six1mla * v1[0] + deuxla * uf[0] ;
    for (n = 1 ; n < nseg ; n++)
    v1[n] = six1mla * v1[n] + deuxla * (uf[n-1] + uf[n]) ;
    v1[nseg] = six1mla * v1[nseg] + deuxla * uf[nsm1] ;
    if (typdeb == 2) v1[1] -= la * uf[0] ;
    if (typfin == 2) v1[nsm1] -= la * uf[nsm1] ;

/* Calcul premiere surdiag */
    for (n = 0 ; n < nseg ; n++)
    v2[n] = six1mla * v2[n] + la * uf[n] ;

/* Calcul deuxieme surdiag */
    for (n = 0 ; n < nsm1 ; n++) v3[n] *= six1mla ;

/*--------------------------------------------------------------------------*/
/*                        Factorisation G = Lt*L                            */
/*--------------------------------------------------------------------------*/
    if (typdeb == 1) ideb = 0 ; else ideb = 1 ;
    if (typfin == 1) ifin = nseg ; else ifin = nsm1 ;
    for (n = ideb ; n < ifin ; n++)
    {
    temp = v2[n]/v1[n] ;
    v1[n+1] -= temp * v2[n] ;
    v2[n+1] -= temp * v3[n] ;
    v2[n] = temp ;
    temp = v3[n]/v1[n] ;
    v1[n+2] -= temp * v3[n] ;
    v3[n] = temp ;
    }

/*--------------------------------------------------------------------------*/
/*   Resolution Lt*L U = QT*M+S  et calcul des coefficients                 */
/*--------------------------------------------------------------------------*/

    if (typdeb == 2) ida = 1 ; else ida = 0 ;
    if (typfin == 2) ifa = nsm1 ; else ifa = nseg ;
    for (i = 0 ; i < dim ; i++)
    {
/*                            CALCUL QT*M + S                               */
    bid = (A[1][i]-A[0][i])/ uf[0] ;
    if (typdeb == 1) qtm[0] = bid - B[0][i] ;
    /* else qtm[0] = 0. ; inutile */
    for (n = 1 ; n < nseg ; n++)
    {
        temp = (A[n+1][i] - A[n][i])/ uf[n] ;
        qtm[n] = temp - bid ;
        bid = temp ;
    }
    if (typfin == 1) qtm[nseg] = - bid + B[nseg][i] ;
    /*  Verifier ci-dessus le _bid ^^^    */
    /* else qtm[nseg] = 0. ; inutile */
    if (typdeb == 2) qtm[1] += (A[0][i]- A[1][i])/ uf[0]
                    + B[0][i] + uf[0] * C[0][i]/3. ;
    if (typfin == 2)
        qtm[nsm1] += (A[nseg][i] - A[nsm1][i])/ uf[nsm1]
                - B[nseg][i] + uf[nsm1] * C[nseg][i]/3. ;

/*                        Resolution Lt*L U = QT*Y+S                        */

    /* Substitution avant */
    ui[ideb] = qtm[ideb] ;
    ui[ideb+1] = qtm[ideb+1] - v2[ideb] * ui[ideb] ;
    for (n = ideb+2 ; n <= ifin ; n++)
        ui[n] = qtm[n] - v2[n-1] * ui[n-1] - v3[n-2]*ui[n-2] ;
    /* Substitution arriere */
    ui[ifin] /= v1[ifin] ;
    ui[ifin-1] = ui[ifin-1]/v1[ifin-1] - v2[ifin-1]*ui[ifin] ;
    for (n = ifin - 2 ; n >= ideb ; n--)
        ui[n] = ui[n]/v1[n] - v2[n]*ui[n+1] - v3[n]*ui[n+2] ;

    /* Ajout u0 et uN si absents */
    if (ideb != 0) ui[0] = 0. ;
    if (ifin != nseg) ui[nseg] = 0. ;
    /* Construction de Q*U dans QTM */
    bid = 0. ;
    for (n = 0 ; n < nseg ; n++)
    {
        temp = (ui[n+1] - ui[n])/ uf[n] ;
        qtm[n] = temp - bid ;
        bid = temp ;
    }
    if (typdeb == 2) qtm[1] -=  ui[1]/ uf[0] ;
    if (typfin == 2) qtm[nsm1] -= ui[nsm1]/ uf[nsm1] ;
    else qtm[nseg] = - temp ;

/* ... Calcul A,B,C,E ... */
    for (n = ida ; n <= ifa; n++)
    {
        A[n][i] -= six1mla * dy[n] * dy[n] * qtm[n] ;
        C[n][i] = troisla * ui[n] ;
    }
    for (n = ida ; n < ifa ; n++)
    {
        temp = uf[n] ;
        bid = (C[n+1][i] - C[n][i]) / 3. ;
        B[n][i] = (A[n+1][i]-A[n][i]) / temp
              - (C[n][i] + bid) * temp ;
        D[n][i] = bid / temp ;
    }
    if (typdeb == 2)
    {
        d = uf[0] ; d2 = d * d ;
        bid = (A[1][i] - A[0][i])/d2 - B[0][i]/d - C[0][i] ;
        temp = (C[1][i] - C[0][i])/3. ;
        D[0][i] = (2. * bid - temp) / d ;
        E[0][i] = ( temp - bid) / d2 ;
    }
    if (typfin == 2)
    {
        d = uf[nsm1] ; d2 = d * d ;
        B[nsm1][i] = 2.*(A[nseg][i] - A[nsm1][i])/d - B[nseg][i]
                      + d*(C[nseg][i] - C[nsm1][i])/3. ;
        bid = (A[nseg][i] - A[nsm1][i])/d2 - B[nsm1][i]/d - C[nsm1][i] ;
        temp = (C[nseg][i] - C[nsm1][i])/3. ;
        D[nsm1][i] = (2. * bid - temp) / d ;
        E[nsm1][i] = ( temp - bid) / d2 ;
    }
    }
    free (qtd_d) ;
}
