/****************************************************************************/
/*     FICHIER TRAJPOL.H : UTILITAIRES POUR TRAJECTOIRES POLYNOMIALES       */
/****************************************************************************/
/* Une trajectoire polynomiale est definie sous la forme suivante :         */
/*                                                                          */
/*         M = A + B u + C u2 + D u3 + E u4 + ---                           */
/*                                                                          */
/*    u est l'abscisse parametrique. Lorsque u vaut 0 le point M est en A   */
/*    Les coefficients A, B, C, D, E, .... ont autant de composantes        */
/*    que la dimension "dim" de l'espace ponctuel de M.                     */
/*                                                                          */
/*    Les coefficients de la polynomiale sont ranges dans un tableau        */
/*    lineaire coef de dim*(degre+1) elements ranges dans l'ordre suivant   */
/*    Ax,Ay,Az,..,Bx,By,Bz,..,Cx,Cy,Cz,..,Dx,Dy,Dz,.......                  */
/*                                                                          */
/****************************************************************************/
#ifndef trajpol_h
#define trajpol_h
/* Dimension max de l'espace des points : a adapter selon les cas           */
#define DIMMAX 10
/* Degre maximun des polynomes : Imperativement >= 5 pour en C2 et 7 en C3  */
#define DEGMAX 7

/****************************************************************************/
/*  UTILITAIRES DE CALCUL DE POINT, DERIVEES, LONGUEUR, POUR POLYNOMIALE    */
/****************************************************************************/
/* POINT d'abscisse parametrique u d'une courbe polynomiale                 */
/*--------------------------------------------------------------------------*/
void ptu_trajpol (double *ptu, double u, double *coef, int dim, int degre) ;

/*--------------------------------------------------------------------------*/
/* DERIVEE parametrique 1ERE d'une courbe polynomiale                       */
/*--------------------------------------------------------------------------*/
void d1Mdu_trajpol (double *d1Mdu, double u, double *coef, int dim, int degre) ;

/*--------------------------------------------------------------------------*/
/* DERIVEE parametrique 2EME d'une courbe polynomiale                       */
/*--------------------------------------------------------------------------*/
void d2Mdu_trajpol (double *d2Mdu, double u, double *coef, int dim, int degre) ;

/*--------------------------------------------------------------------------*/
/* DERIVEE parametrique 3EME d'une courbe polynomiale                       */
/*--------------------------------------------------------------------------*/
void d3Mdu_trajpol (double *d3Mdu, double u, double *coef, int dim, int degre) ;

/*--------------------------------------------------------------------------*/
/* DERIVEE 1ERE abscisse CURVILIGNE / PARAMETRIQUE                          */
/*--------------------------------------------------------------------------*/
double dsdu_trajpol (double u, double *coef, int dim, int degre) ;

/*--------------------------------------------------------------------------*/
/* DERIVEE 2EME abscisse CURVILIGNE /   PARAMETRIQUE                        */
/*--------------------------------------------------------------------------*/
double d2sdu_trajpol (double u, double *coef, int dim, int degre) ;

/*--------------------------------------------------------------------------*/
/* DERIVEE 2EME abscisse PARAMETRIQUE / CURVILIGNE                          */
/*--------------------------------------------------------------------------*/
double d2uds_trajpol (double u, double *coef, int dim, int degre) ;

/*--------------------------------------------------------------------------*/
/* Vecteur tangentiel unitaire en u d'une courbe polynomiale                           */
/*--------------------------------------------------------------------------*/
void d1Mdsu_trajpol (double *dM_ds,
                   double u, double *coef, int dim, int degre) ;

/*--------------------------------------------------------------------------*/
/* Vecteur COURBURE en u d'une courbe polynomiale                           */
/*--------------------------------------------------------------------------*/
void d2Mdsu_trajpol (double *d2M_ds2,
                   double u, double *coef, int dim, int degre) ;

/*--------------------------------------------------------------------------*/
/* Vecteur d3M/ds3 en u d'une courbe polynomiale                           */
/*--------------------------------------------------------------------------*/
void d3Mdsu_trajpol (double *d3M_ds3,
                   double u, double *coef, int dim, int degre) ;

/*--------------------------------------------------------------------------*/
/* LONGUEUR d'un arc de courbe polynomiale                                  */
/*--------------------------------------------------------------------------*/
double u2s_trajpol(double uf, double *coef, int dim, int degre, int niss) ;

/*--------------------------------------------------------------------------*/
/* Calcul u correspondant a s par interpolation                             */
/*--------------------------------------------------------------------------*/
double s2u_trajpol (double s, double uf, double *coef, int dim, int degre,
                        int niss) ;

/*--------------------------------------------------------------------------*/
/* Position du CENTRE DE COURBURE par rapport au point courant              */
/*--------------------------------------------------------------------------*/
double d2Mds_2_MC (double *vec, int dim) ;

/*--------------------------------------------------------------------------*/
/* LONGUEUR curviligne approximative d'une trajectoire                      */
/*--------------------------------------------------------------------------*/
double long_estim (double *p1, double *p2, double *t1, double *t2, int dim) ;

/*--------------------------------------------------------------------------*/
/* DECALAGE ORIGINE abscisse parametrique traj. polynomiale                 */
/*--------------------------------------------------------------------------*/
void dori_trajpol (double *coef, int dim, int degre, double epsu) ;

/****************************************************************************/
/* CALCUL DE RACCORDS CONTINUS D'ORDRE 2 PAR TRAJECTOIRES POLYNOMIALIRES    */
/****************************************************************************/
/*--------------------------------------------------------------------------*/
/* RACCORD 2 points par une droite                                          */
/*--------------------------------------------------------------------------*/
void trajpol_drte00 (double *A1, double Uf, double *A2, int dim) ;

/*--------------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une parabole (b2 donne)                    */
/*--------------------------------------------------------------------------*/
void trajpol_bole01(double *A1, double d1, double *A2, int n) ;

/*--------------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une parabole (b1 donne)                    */
/*--------------------------------------------------------------------------*/
void trajpol_bole10(double *A1, double d1, double *A2, int n) ;

/*--------------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une cubique (b2,c2 donnes)                 */
/*--------------------------------------------------------------------------*/
void trajpol_cubi02(double *A1, double d1, double *A2, int n) ;

/*--------------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une cubique (b1 et b2 donnes)              */
/*--------------------------------------------------------------------------*/
void trajpol_cubi11 (double *A1, double d, double *A2, int n) ;

/*--------------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une cubique (b1,c1 donnes)                 */
/*--------------------------------------------------------------------------*/
void trajpol_cubi20(double *A1, double d1, double *A2, int n) ;

/*--------------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une quadrique (b1 et b2,c2 donnes)         */
/*--------------------------------------------------------------------------*/
void trajpol_quad12(double *A1, double d, double *A2, int n) ;

/*--------------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par une quadrique (b1,c1 et b2 donnes)         */
/*--------------------------------------------------------------------------*/
void trajpol_quad21(double *A1, double d, double *A2, int n) ;

/*--------------------------------------------------------------------------*/
/* RACCORD c2 entre 2 points par un polfive (b1,c1 et b2,c2 donnes)         */
/*--------------------------------------------------------------------------*/
void trajpol_five22 (double *A1, double d, double *A2, int n) ;

/*--------------------------------------------------------------------------*/
/* RACCORD c2 entre 3 points par 2 paraboles                                */
/*--------------------------------------------------------------------------*/
void trajpol_2boles00 (double *A1, double d1, double *A2, double d2,
                            double *A3, int n) ;

/*--------------------------------------------------------------------------*/
/* RACCORD c2 entre 3 points par 2 quadriques (b1,c1 et b3,c3 donnes)       */
/*--------------------------------------------------------------------------*/
void trajpol_2quads22 (double *A1, double d1, double *A2, double d2,
                            double *A3, int n) ;

/*--------------------------------------------------------------------------*/
/* CONTRAINTES DE RACCORD AUX EXTREMITES (pour typdeb ou typfin)            */
/*--------------------------------------------------------------------------*/
#define CRNUL -1                        /* Courbure nulle imposee au bout   */
#define NEANT 0                         /* Rien d'impose au bout            */
#define NDFIC 0                         /* Traite en noeud fictif           */
#define TGTE  1                         /* Tangente impose au bout         */
#define QUADRI 2                        /* Tgte et courbure imposee au bout */
/*--------------------------------------------------------------------------*/
/* SPLINE CUBIQUE classique  avec #define ci-dessus pour typdeb et typfin   */
/*--------------------------------------------------------------------------*/
void trisplin (double **A, double *d, int typdeb, int typfin, int NBSG,
                                int NBCOMP) ;
/*--------------------------------------------------------------------------*/
/*
    ENTREES : A, d, typdeb, typfin, NBSG, NBCOMP
    A est un pointeur sur NBSG+1 blocs de doubles.
        Dans chaque bloc il y au moins la place pour 4 coefficients
        avec NBCOMP composantes,
    d est le tableau des valeurs finales
        des paramtres des NBSG+1 segments (dernier bidon mais utilise).

    typdeb decrit le type de raccord en debut de trajectoire :
        0 ou
        NDFIC : rien specifie
        CRNUL : pas de tangente et courbure nulle au depart
        1 : tangente donnee
        2 : raccord par quadrique
    typfin decrit le type de raccord en fin de trajectoire (idem typdeb)
    SORTIES : B, C, D calculees dans les tableaux pointes par A
*/
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* SPLINE CUBIQUE lissante  avec #define ci-dessus pour typdeb et typfin   */
/*--------------------------------------------------------------------------*/
void lispline (double **coefs, double *uf, int typdeb, int typfin, int nseg,
                         int dim, double *dy, double la) ;
/*--------------------------------------------------------------------------*/
/*
    ENTREES : idem trisplin : coefs, uf, typdeb, typfin, nseg, dim
    coefs est un pointeur sur NBSG+1 blocs de doubles.
          Dans chaque bloc il y au moins la place pour 4 coefficients
          avec NBCOMP composantes, sauf le dernier (NBSG+1) qui peut ne
          contenir la place que pour 3 coefficients.
    uf    est le tableau des valeurs finales
          des paramtres des NBSG+1 segments (dernier bidon mais utilise).
    typdeb decrit le type de raccord en debut de trajectoire :
    typfin decrit le type de raccord en fin de trajectoire (idem typdeb)
    nseg  est le nombre de segments
    dim   est la dimension de l'espace ponctuel de la trajectoire
    En plus de trisplin :
    dy    est le tableau des tolerances aux nseg+1 points
    la    est le coefficient de filtrage compris entre 0. et 1.
          0. : pas de filtrage : passage exact par les points
          1.  : filtrage maximal
    SORTIES : Coefficients polynomiaux dans les tableaux pointes par coefs
*/
/*--------------------------------------------------------------------------*/
#endif
